import os
import sys

from coleo import Argument, auto_cli
from pystache import Renderer
from pystache.context import KeyNotFoundError


render = Renderer(missing_tags="strict")


def transform(path, text, env):
    try:
        return render.render(text, env)
    except KeyNotFoundError as err:
        print(
            f"Key '{err.args[0]}' required for file '{path}' was not found",
            file=sys.stderr,
        )
        sys.exit(1)


def run():
    """Copy a template file or directory."""

    # [positional]
    # Path to the file or directory to copy
    path: Argument

    # [positional: *]
    # key=value pairs to substitute
    env: Argument
    env_dict = dict(k.split("=") for k in env)

    full_path = os.path.abspath(path)

    if not os.path.exists(full_path):
        print(f"Error: {full_path} does not exist", file=sys.stderr)
        sys.exit(1)

    base = os.path.dirname(full_path)
    if not base.endswith("/"):
        base += "/"
    base_length = len(base)

    new_base = "."

    gen = {}

    data = list(os.walk(full_path, topdown=True))
    if os.path.isdir(full_path):
        first_entry = (base, [os.path.basename(full_path)], [])
    else:
        first_entry = (base, [], [os.path.basename(full_path)])
    data.insert(0, first_entry)
    for dirname, dirs, files in data:
        for d in dirs:
            full = os.path.join(dirname, d)
            relative = full[base_length:]
            new_relative = transform(full, relative, env_dict)
            gen[os.path.join(new_base, new_relative)] = None

        for f in files:
            full = os.path.join(dirname, f)
            relative = full[base_length:]
            new_relative = transform(relative, relative, env_dict)
            gen[os.path.join(new_base, new_relative)] = transform(
                full, open(full).read(), env_dict
            )

    for filename, contents in gen.items():
        if contents is None:
            print(f"Generating directory: {filename}")
            os.makedirs(filename)
        else:
            print(f"Generating file: {filename}")
            open(filename, "w").write(contents)


def main():
    auto_cli(run, [])



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['cp_template']
install_requires = \
['coleo>=0.1.3,<0.2.0', 'pystache>=0.5.4,<0.6.0']

entry_points = \
{'console_scripts': ['cp-template = cp_template:main']}

setup_kwargs = {
    'name': 'cp-template',
    'version': '0.1.0',
    'description': 'A tool to copy templated directories',
    'long_description': '\n# cp-template\n\n\nThis is a very simple utility to generate directories based on templates.\n\n\n## Install\n\n\n```bash\npip install cp-template\n```\n\n\n## Usage\n\n\nSuppose you have the following directory structure (**Note: the {{}}s are part of the filenames**)\n\n```\n{{project}}/\n  .gitignore\n  README.md        # File contains "{{project}} by {{author}}"\n  {{project}}/\n    __init__.py\n```\n\nThen you can run the following command:\n\n```\ncp-template \'./{{project}}\' project=pineapple author=me\n```\n\nAnd it will generate this in the current directory:\n\n```\npineapple/\n  .gitignore\n  README.md        # File contains "pineapple by me"\n  pineapple/\n    __init__.py\n```\n\nMore features will be added as I need them, but feel free to make PRs to contribute some.\n',
    'author': 'Olivier Breuleux',
    'author_email': 'breuleux@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/breuleux/cp-template',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
